@@ -1,4 +1,4 @@
-from fakeredis.stack import JSONCommandsMixin, BFCommandsMixin
+from fakeredis.stack import JSONCommandsMixin, BFCommandsMixin, CFCommandsMixin, CMSCommandsMixin, TopkCommandsMixin
 from ._basefakesocket import BaseFakeSocket
 from .commands_mixins.bitmap_mixin import BitmapCommandsMixin
 from .commands_mixins.connection_mixin import ConnectionCommandsMixin
@@ -7,7 +7,14 @@ from .commands_mixins.geo_mixin import GeoCommandsMixin
 from .commands_mixins.hash_mixin import HashCommandsMixin
 from .commands_mixins.list_mixin import ListCommandsMixin
 from .commands_mixins.pubsub_mixin import PubSubCommandsMixin
-from .commands_mixins.scripting_mixin import ScriptingCommandsMixin
+
+try:
+    from lupa import LuaRuntime  # noqa: F401
+    from .commands_mixins.scripting_mixin import ScriptingCommandsMixin
+except ImportError:
+    class ScriptingCommandsMixin:
+        pass
+
 from .commands_mixins.server_mixin import ServerCommandsMixin
 from .commands_mixins.set_mixin import SetCommandsMixin
 from .commands_mixins.sortedset_mixin import SortedSetCommandsMixin
@@ -34,6 +41,9 @@ class FakeSocket(
     JSONCommandsMixin,
     GeoCommandsMixin,
     BFCommandsMixin,
+    CFCommandsMixin,
+    CMSCommandsMixin,
+    TopkCommandsMixin,
 ):
     def __init__(self, server, db):
         super(FakeSocket, self).__init__(server, db)
