@@ -1,10 +1,11 @@
 import bisect
 import itertools
+import sys
 import time
 from collections import Counter
 from dataclasses import dataclass
 from operator import itemgetter
-from typing import List, Union, Tuple, Optional, NamedTuple, Dict, Any
+from typing import List, Union, Tuple, Optional, NamedTuple, Dict, Any, Sequence
 
 from fakeredis._commands import BeforeAny, AfterAny
 from fakeredis._helpers import current_time
@@ -15,21 +16,25 @@ class StreamEntryKey(NamedTuple):
     seq: int
 
     def encode(self) -> bytes:
-        return f'{self.ts}-{self.seq}'.encode()
+        return f"{self.ts}-{self.seq}".encode()
 
     @staticmethod
-    def parse_str(entry_key_str: Union[bytes, str]) -> 'StreamEntryKey':
+    def parse_str(entry_key_str: Union[bytes, str]) -> "StreamEntryKey":
         if isinstance(entry_key_str, bytes):
             entry_key_str = entry_key_str.decode()
-        s = entry_key_str.split('-')
-        (timestamp, sequence) = (int(s[0]), 0) if len(s) == 1 else (int(s[0]), int(s[1]))
+        s = entry_key_str.split("-")
+        (timestamp, sequence) = (
+            (int(s[0]), 0) if len(s) == 1 else (int(s[0]), int(s[1]))
+        )
         return StreamEntryKey(timestamp, sequence)
 
 
 class StreamRangeTest:
     """Argument converter for sorted set LEX endpoints."""
 
-    def __init__(self, value: Union[StreamEntryKey, BeforeAny, AfterAny], exclusive: bool):
+    def __init__(
+            self, value: Union[StreamEntryKey, BeforeAny, AfterAny], exclusive: bool
+    ):
         self.value = value
         self.exclusive = exclusive
 
@@ -43,11 +48,11 @@ class StreamRangeTest:
 
     @classmethod
     def decode(cls, value: bytes, exclusive=False):
-        if value == b'-':
+        if value == b"-":
             return cls(BeforeAny(), True)
-        elif value == b'+':
+        elif value == b"+":
             return cls(AfterAny(), True)
-        elif value[:1] == b'(':
+        elif value[:1] == b"(":
             return cls(StreamEntryKey.parse_str(value[1:]), True)
         return cls(StreamEntryKey.parse_str(value), exclusive)
 
@@ -66,17 +71,27 @@ class StreamConsumerInfo(object):
         self.last_attempt = _time
         self.last_success = _time
 
-    def info(self, curr_time: int) -> List[bytes]:
+    def info(self, curr_time: int) -> List[Union[bytes, int]]:
         return [
-            b'name', self.name,
-            b'pending', self.pending,
-            b'idle', curr_time - self.last_attempt,
-            b'inactive', curr_time - self.last_success,
+            b"name",
+            self.name,
+            b"pending",
+            self.pending,
+            b"idle",
+            curr_time - self.last_attempt,
+            b"inactive",
+            curr_time - self.last_success,
         ]
 
 
 class StreamGroup(object):
-    def __init__(self, stream: 'XStream', name: bytes, start_key: StreamEntryKey, entries_read: int = None):
+    def __init__(
+            self,
+            stream: "XStream",
+            name: bytes,
+            start_key: StreamEntryKey,
+            entries_read: Optional[int] = None,
+    ):
         self.stream = stream
         self.name = name
         self.start_key = start_key
@@ -87,15 +102,16 @@ class StreamGroup(object):
         self.last_ack_key = start_key
         # Pending entry List, see https://redis.io/commands/xreadgroup/
         # msg_id -> consumer_name, time read
-        self.pel = dict()
+        self.pel: Dict[StreamEntryKey, Any] = dict()
 
-    def set_id(self, last_delivered_str: bytes, entries_read: Union[int, None]) -> None:
-        """Set last_delivered_id for the group
-        """
+    def set_id(self, last_delivered_str: bytes, entries_read: Optional[int]) -> None:
+        """Set last_delivered_id for the group"""
         self.start_key = self.stream.parse_ts_seq(last_delivered_str)
         start_index, _ = self.stream.find_index(self.start_key)
-        self.entries_read = entries_read
-        self.last_delivered_key = self.stream[min(start_index + (entries_read or 0), len(self.stream) - 1)]
+        self.entries_read = entries_read or 0
+        self.last_delivered_key = self.stream[
+            min(start_index + (entries_read or 0), len(self.stream) - 1)
+        ]
 
     def add_consumer(self, consumer_name: bytes) -> int:
         if consumer_name in self.consumers:
@@ -118,26 +134,28 @@ class StreamGroup(object):
         last_delivered_index, _ = self.stream.find_index(self.last_delivered_key)
         last_ack_index, _ = self.stream.find_index(self.last_ack_key)
         if start_index + (self.entries_read or 0) > len(self.stream):
-            lag = len(self.stream) - start_index - self.entries_read
+            lag = len(self.stream) - start_index - (self.entries_read or 0)
         else:
             lag = len(self.stream) - 1 - last_delivered_index
         res = {
-            b'name': self.name,
-            b'consumers': len(self.consumers),
-            b'pending': last_delivered_index - last_ack_index,
-            b'last-delivered-id': self.last_delivered_key.encode(),
-            b'entries-read': self.entries_read,
-            b'lag': lag,
+            b"name": self.name,
+            b"consumers": len(self.consumers),
+            b"pending": last_delivered_index - last_ack_index,
+            b"last-delivered-id": self.last_delivered_key.encode(),
+            b"entries-read": self.entries_read,
+            b"lag": lag,
         }
-        return list(itertools.chain(*res.items()))
+        return list(itertools.chain(*res.items()))  # type: ignore
 
-    def group_read(self, consumer_name: bytes, start_id: bytes, count: int, noack: bool) -> List:
+    def group_read(
+            self, consumer_name: bytes, start_id: bytes, count: int, noack: bool
+    ) -> List:
         _time = current_time()
         if consumer_name not in self.consumers:
             self.consumers[consumer_name] = StreamConsumerInfo(consumer_name)
 
         self.consumers[consumer_name].last_attempt = _time
-        if start_id == b'>':
+        if start_id == b">":
             start_key = self.last_delivered_key
         else:
             start_key = max(StreamEntryKey.parse_str(start_id), self.last_delivered_key)
@@ -153,7 +171,10 @@ class StreamGroup(object):
         return [self.stream.format_record(x) for x in ids_read]
 
     def _calc_consumer_last_time(self):
-        new_last_success_map = {k: min(v)[1] for k, v in itertools.groupby(self.pel.values(), key=itemgetter(0))}
+        new_last_success_map = {
+            k: min(v)[1]
+            for k, v in itertools.groupby(self.pel.values(), key=itemgetter(0))
+        }
         for consumer in new_last_success_map:
             if consumer not in self.consumers:
                 self.consumers[consumer] = StreamConsumerInfo(consumer)
@@ -175,33 +196,35 @@ class StreamGroup(object):
         self._calc_consumer_last_time()
         return res
 
-    def pending(self, idle: Optional[int],
-                start: Optional[StreamRangeTest],
-                end: Optional[StreamRangeTest],
-                count: Optional[int],
-                consumer: Optional[str]):
+    def pending(
+            self,
+            idle: Optional[int],
+            start: Optional[StreamRangeTest],
+            end: Optional[StreamRangeTest],
+            count: Optional[int],
+            consumer: Optional[str],
+    ):
         _time = current_time()
         relevent_ids = list(self.pel.keys())
         if consumer is not None:
-            relevent_ids = [k for k in relevent_ids
-                            if self.pel[k][0] == consumer]
+            relevent_ids = [k for k in relevent_ids if self.pel[k][0] == consumer]
         if idle is not None:
-            relevent_ids = [
-                k for k in relevent_ids
-                if self.pel[k][1] + idle < _time
-            ]
+            relevent_ids = [k for k in relevent_ids if self.pel[k][1] + idle < _time]
         if start is not None and end is not None:
             relevent_ids = [
-                k for k in relevent_ids
-                if (((start.value < k) or (start.value == k and not start.exclusive))
-                    and ((end.value > k) or (end.value == k and not end.exclusive)))
+                k
+                for k in relevent_ids
+                if (
+                        ((start.value < k) or (start.value == k and not start.exclusive))
+                        and ((end.value > k) or (end.value == k and not end.exclusive))
+                )
             ]
         if count is not None:
             relevent_ids = sorted(relevent_ids)[:count]
 
         return [[k.encode(), self.pel[k][0]] for k in relevent_ids]
 
-    def pending_summary(self) -> List[bytes]:
+    def pending_summary(self) -> List[Any]:
         counter = Counter([self.pel[k][0] for k in self.pel])
         data = [
             len(self.pel),
@@ -211,16 +234,20 @@ class StreamGroup(object):
         ]
         return data
 
-    def claim(self,
-              min_idle_ms: int,
-              msgs: List[bytes],
-              consumer_name: bytes,
-              _time: Optional[int],
-              force: bool) -> Tuple[List, List]:
+    def claim(
+            self,
+            min_idle_ms: int,
+            msgs: Sequence[bytes],
+            consumer_name: bytes,
+            _time: Optional[int],
+            force: bool,
+    ) -> Tuple[List, List]:
         curr_time = current_time()
         if _time is None:
             _time = curr_time
-        self.consumers.get(consumer_name, StreamConsumerInfo(consumer_name)).last_attempt = curr_time
+        self.consumers.get(
+            consumer_name, StreamConsumerInfo(consumer_name)
+        ).last_attempt = curr_time
         claimed_msgs, deleted_msgs = [], []
         for msg in msgs:
             try:
@@ -250,9 +277,13 @@ class StreamGroup(object):
     def read_pel_msgs(self, min_idle_ms: int, start: bytes, count: int):
         start_key = StreamEntryKey.parse_str(start)
         curr_time = current_time()
-        msgs = sorted([k for k in self.pel
-                       if (curr_time - self.pel[k][1] >= min_idle_ms)
-                       and k >= start_key])
+        msgs = sorted(
+            [
+                k
+                for k in self.pel
+                if (curr_time - self.pel[k][1] >= min_idle_ms) and k >= start_key
+            ]
+        )
         count = min(count, len(msgs))
         return msgs[:count]
 
@@ -281,14 +312,16 @@ class XStream:
     def group_get(self, group_name: bytes) -> Optional[StreamGroup]:
         return self._groups.get(group_name, None)
 
-    def group_add(self, name: bytes, start_key_str: bytes, entries_read: Union[int, None]) -> None:
+    def group_add(
+            self, name: bytes, start_key_str: bytes, entries_read: Optional[int]
+    ) -> None:
         """Add a group listening to stream
 
         :param name: Group name
         :param start_key_str: start_key in `timestamp-sequence` format, or $ listen from last.
         :param entries_read: Number of entries read.
         """
-        if start_key_str == b'$':
+        if start_key_str == b"$":
             start_key = self._ids[-1] if len(self._ids) > 0 else StreamEntryKey(0, 0)
         else:
             start_key = StreamEntryKey.parse_str(start_key_str)
@@ -309,18 +342,24 @@ class XStream:
 
     def stream_info(self, full: bool) -> List[bytes]:
         res = {
-            b'length': len(self._ids),
-            b'groups': len(self._groups),
-            b'first-entry': self.format_record(self._ids[0]) if len(self._ids) > 0 else None,
-            b'last-entry': self.format_record(self._ids[-1]) if len(self._ids) > 0 else None,
-            b'max-deleted-entry-id': self._max_deleted_id.encode(),
-            b'entries-added': self._entries_added,
-            b'recorded-first-entry-id': self._ids[0].encode() if len(self._ids) > 0 else b'0-0',
+            b"length": len(self._ids),
+            b"groups": len(self._groups),
+            b"first-entry": self.format_record(self._ids[0])
+            if len(self._ids) > 0
+            else None,
+            b"last-entry": self.format_record(self._ids[-1])
+            if len(self._ids) > 0
+            else None,
+            b"max-deleted-entry-id": self._max_deleted_id.encode(),
+            b"entries-added": self._entries_added,
+            b"recorded-first-entry-id": self._ids[0].encode()
+            if len(self._ids) > 0
+            else b"0-0",
         }
         if full:
-            res[b'entries'] = [self.format_record(i) for i in self._ids]
-            res[b'groups'] = [g.group_info() for g in self._groups.values()]
-        return list(itertools.chain(*res.items()))
+            res[b"entries"] = [self.format_record(i) for i in self._ids]
+            res[b"groups"] = [g.group_info() for g in self._groups.values()]
+        return list(itertools.chain(*res.items()))  # type: ignore
 
     def delete(self, lst: List[Union[str, bytes]]) -> int:
         """Delete items from stream
@@ -338,10 +377,10 @@ class XStream:
                 res += 1
         return res
 
-    def add(self, fields: List, entry_key: str = '*') -> Union[None, bytes]:
+    def add(self, fields: List, entry_key: str = "*") -> Union[None, bytes]:
         """Add entry to a stream.
 
-        If the entry_key can not be added (because its timestamp is before the last entry, etc.),
+        If the entry_key cannot be added (because its timestamp is before the last entry, etc.),
         nothing is added.
 
         :param fields: List of fields to add, must [key1, value1, key2, value2, ... ]
@@ -360,18 +399,20 @@ class XStream:
         if isinstance(entry_key, bytes):
             entry_key = entry_key.decode()
 
-        if entry_key is None or entry_key == '*':
+        if entry_key is None or entry_key == "*":
             ts, seq = int(1000 * time.time()), 0
-            if (len(self._ids) > 0
+            if (
+                    len(self._ids) > 0
                     and self._ids[-1].ts == ts
-                    and self._ids[-1].seq >= seq):
+                    and self._ids[-1].seq >= seq
+            ):
                 seq = self._ids[-1].seq + 1
             ts_seq = StreamEntryKey(ts, seq)
-        elif entry_key[-1] == '*':  # entry_key has `timestamp-*` structure
-            split = entry_key.split('-')
+        elif entry_key[-1] == "*":  # entry_key has `timestamp-*` structure
+            split = entry_key.split("-")
             if len(split) != 2:
                 return None
-            ts, seq = int(split[0]), split[1]
+            ts, seq = int(split[0]), split[1]  # type: ignore
             if len(self._ids) > 0 and ts == self._ids[-1].ts:
                 seq = self._ids[-1].seq + 1
             else:
@@ -423,53 +464,58 @@ class XStream:
             ind = bisect.bisect_right(self._ids, entry_key)
         return ind, (ind < len(self._ids) and self._ids[ind] == entry_key)
 
-    def find_index_key_as_str(self, entry_key_str: Union[str, bytes]) -> Tuple[int, bool]:
+    def find_index_key_as_str(
+            self, entry_key_str: Union[str, bytes]
+    ) -> Tuple[int, bool]:
         """Find the closest index to entry_key_str in the stream
         :param entry_key_str: key for the entry, formatted as 'timestamp-sequence.'
         :returns: A tuple
             (index of entry with the closest (from the left) key to entry_key_str,
              whether the entry key is equal)
         """
-        if entry_key_str == b'$':
+        if entry_key_str == b"$":
             return max(len(self._ids) - 1, 0), True
         ts_seq = StreamEntryKey.parse_str(entry_key_str)
         return self.find_index(ts_seq)
 
     @staticmethod
     def parse_ts_seq(ts_seq_str: Union[str, bytes]) -> StreamEntryKey:
-        if ts_seq_str == b'$':
+        if ts_seq_str == b"$":
             return StreamEntryKey(0, 0)
         return StreamEntryKey.parse_str(ts_seq_str)
 
-    def trim(self,
-             max_length: Optional[int] = None,
-             start_entry_key: Optional[str] = None,
-             limit: Optional[int] = None) -> int:
+    def trim(
+            self,
+            max_length: Optional[int] = None,
+            start_entry_key: Optional[str] = None,
+            limit: Optional[int] = None,
+    ) -> int:
         """Trim a stream
 
         :param max_length: Max length of the resulting stream after trimming (number of last values to keep)
-        :param start_entry_key: Min entry-key to keep, can not be given together with max_length.
+        :param start_entry_key: Min entry-key to keep, cannot be given together with max_length.
         :param limit: Number of entries to keep from minid.
         :returns: The resulting stream after trimming.
         :raises ValueError: When both max_length and start_entry_key are passed.
         """
         if max_length is not None and start_entry_key is not None:
-            raise ValueError('Can not use both max_length and start_entry_key')
-        start_ind = None
+            raise ValueError("Can not use both max_length and start_entry_key")
+        start_ind: Optional[int] = None
         if max_length is not None:
             start_ind = len(self._ids) - max_length
         elif start_entry_key is not None:
             ind, exact = self.find_index_key_as_str(start_entry_key)
             start_ind = ind
-        res = max(start_ind, 0)
-        if limit is not None:
-            res = min(start_ind, limit)
+        res: int = min(max(start_ind or 0, 0), limit or sys.maxsize)
+
         remove_keys, self._ids = self._ids[:res], self._ids[res:]
         for k in remove_keys:
             del self._values_dict[k]
         return res
 
-    def irange(self, start: StreamRangeTest, stop: StreamRangeTest, reverse=False) -> List[Any]:
+    def irange(
+            self, start: StreamRangeTest, stop: StreamRangeTest, reverse=False
+    ) -> List[Any]:
         """Returns a range of the stream values from start to stop.
 
         :param start: Start key
@@ -490,21 +536,29 @@ class XStream:
 
         start_ind = _find_index(start)
         stop_ind = _find_index(stop, from_left=False)
-        matches = map(lambda x: self.format_record(self._ids[x]), range(start_ind, stop_ind))
+        matches = map(
+            lambda x: self.format_record(self._ids[x]), range(start_ind, stop_ind)
+        )
         if reverse:
             return list(reversed(tuple(matches)))
         return list(matches)
 
     def last_item_key(self) -> bytes:
-        return self._ids[-1].encode() if len(self._ids) > 0 else '0-0'.encode()
+        return self._ids[-1].encode() if len(self._ids) > 0 else "0-0".encode()
 
-    def stream_read(self, start_key: StreamEntryKey, count: Union[int, None]) -> List[StreamEntryKey]:
+    def stream_read(
+            self, start_key: StreamEntryKey, count: Union[int, None]
+    ) -> List[StreamEntryKey]:
         start_ind, found = self.find_index(start_key)
         if found:
             start_ind += 1
         if start_ind >= len(self):
             return []
-        end_ind = len(self) if count is None or start_ind + count >= len(self) else start_ind + count
+        end_ind = (
+            len(self)
+            if count is None or start_ind + count >= len(self)
+            else start_ind + count
+        )
         return self._ids[start_ind:end_ind]
 
     def format_record(self, key: StreamEntryKey) -> List:
